// searchit.cpp - Plugin for mado-tsukai-no-yuuutsu.
// Searches string by various search engine.

/*
Copyright (c) 2007, Canada, Masakatz <canada-nodokal@kaede.to> All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the 
following conditions are met:

    * Redistributions of source code must retain the above
	  copyright notice, this list of conditions and the 
	  following disclaimer.
    * Redistributions in binary form must reproduce the 
	  above copyright notice, this list of conditions and
	  the following disclaimer in the documentation and/or 
	  other materials provided with the distribution.
    * Any the names of its contributors may not be used to 
	  endorse or promote products derived from this software
	  without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "stdafx.h"
#include "shellapi.h"
#include <stdio.h>
#include "tools.h"

void searchit_init(void);

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			searchit_init();
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

#define URLSIZE 256
#define BUFSIZE 1024

static char user_url[26][URLSIZE];
static char user_enc[26];

int isblank(int c){return (c == ' ' || c == '\t') ? true : false;}

void searchit_init(void){
	FILE *fd;
	char buf[BUFSIZE]; // in case of..
	int count    = 0;
	int user_def = 0;
	char enc;
	int error    = 0;
	char *bufptr;
	char *urlptr;
	int lines    = 0;

	const _TCHAR *buff_env;
	TCHAR buff_path[MAX_PATH * 4] = _T("\0");

	// initialize..
	for(count = 0; count < 26; count++){
		user_url[count][0] = '\0';
	}

	fd = fopen("Plugins\\searchit.txt", "r");
	if(fd == NULL)
		{
		fd = fopen("Plugin\\searchit.txt", "r");
		if(fd == NULL)
			{
			buff_env = _tgetenv(_T("USERPROFILE"));
			if(buff_env)
				{
				sprintf(buff_path, "%s\\searchit.txt", buff_env);
				OutputDebugString(buff_path);
				fd = fopen(buff_path, "r");
				if(fd == NULL)
					{
					return;
					}
				else
					{
					goto fd_ok;
					}
				}
			return;
			}
		}

fd_ok:
	while(fgets(buf, BUFSIZE, fd)){
		lines++;
		bufptr = buf;
		// First char must be between 'a' - 'z', or entire line is ignored.
		if(! islower(*bufptr)) continue;
		user_def = *bufptr - 'a'; // 'a' = 0, 'b' = 1, 'c' = 2...

		// Second char must be blank(space or tab).
		if(! isblank(*++bufptr)) continue;
		while(isblank(*bufptr)) bufptr++; // Skip blanks.

		// Encode type must be one of 'S', 'U', 'E'.
		if(*bufptr != 'S' && *bufptr != 'U' && *bufptr != 'E'){
			error++;
			break;
		}
		enc = *bufptr;

		// Skip blanks.
		if(! isblank(*++bufptr)){
			error++;
			break;
		}
		while(isblank(*bufptr)) bufptr++;

		// Easy check for url.
		if( ! strncmp(bufptr, "http://" , 7)
		 && ! strncmp(bufptr, "https://", 8)
		 && ! strncmp(bufptr, "shttp://", 8)
		){
			error++;
			break;
		}
		
		urlptr = bufptr;

		while(*bufptr){
			if(*bufptr > '\x7E' || *bufptr < '\x21'){
				*bufptr = '\0';
				break;
			}
			bufptr++;
		}

		int urllen = strlen(urlptr);

		// URL must be less than 256bytes.
		if(urllen >= URLSIZE || urllen == 0){
			error++;
			break;
		}

		strncpy(user_url[user_def], urlptr, URLSIZE);
		user_url[user_def][URLSIZE - 1] = '\0'; // In case of..
		user_enc[user_def] = enc;
	}
	fclose(fd);
	char errmsg[256];

	if(error){
		sprintf(errmsg, "searchit.txt: Syntax error at %d", lines);
		MessageBox(0, errmsg, 0, MB_OK);
		return;
	}

}

// Trim whitespaces at end of string.
void trim(char *totrim){
	int trim_len = strlen(totrim);

	while(trim_len--){
		if(totrim[trim_len] == ' ' || totrim[trim_len] == '\t')
			totrim[trim_len] = '\0';
		else break;
	}
}

void searchit(char *search_url, const wchar_t *search_for_w, char enctype){
	char buffer[(BUFSIZE)];
	char search_for[(BUFSIZE)];
	char search_for_sjis[(BUFSIZE)];
	int url_len;
	int result;

	*search_for = '\0';

	// Make 'search_for' from 'search_for_w' in 'enctype' encoding.
	switch(enctype){
	case 'S': // Shift_JIS
		WideCharToMultiByte(0, 0, search_for_w, -1, search_for, (BUFSIZE), 0, 0);
		break;
	case 'U': // UTF-8
		// utf162utf8 is at tools.cpp
		utf162utf8(search_for, search_for_w, (BUFSIZE));
		break;
	case 'E': // EUC-JP
		result = WideCharToMultiByte(0, 0, search_for_w, -1, search_for_sjis, (BUFSIZE), 0, 0);
		// search_for remains null string if it fails.
		if(result == 0) break;
		// sjis2euc_jp is at tools.cpp
		sjis2euc_jp((unsigned char *)search_for, (const unsigned char *)search_for_sjis, strlen(search_for_sjis));
		break;
	case 'L': // UTF-16 to ISO 8859-1 Latin 1
		utf16toLatin1(search_for, search_for_w, (BUFSIZE));
		break;
	default:
		break;
	}
	
	// Trim whitespaces at end of string.
	trim(search_for);

	url_len = strlen(search_url);

	if(url_len >= (BUFSIZE)) return; // just in case.

	// Use null string if URL is too long.
	if(url_len + strlen(search_for) * 3 >= (BUFSIZE)) 
		*search_for = '\0';

	// Attach encoded 'search_for' string to 'search_url'.
	strcpy(buffer, search_url);
	UrlEncode(buffer + url_len, search_for);

	// Open window for the URL.
	ShellExecute(NULL, "open", buffer, NULL, NULL, SW_SHOWNORMAL);
}

void searchit2(char *search_url, char *add_url, const wchar_t *search_for_w, char enctype){
	char buffer[(BUFSIZE)];
	char buffer2[(BUFSIZE)];
	char search_for[(BUFSIZE)];
	char search_for_sjis[(BUFSIZE)];
	int url_len, url_len2;
	int result;

	*search_for = '\0';

	// Make 'search_for' from 'search_for_w' in 'enctype' encoding.
	switch(enctype){
	case 'S': // Shift_JIS
		WideCharToMultiByte(0, 0, search_for_w, -1, search_for, (BUFSIZE), 0, 0);
		break;
	case 'U': // UTF-8
		// utf162utf8 is at tools.cpp
		utf162utf8(search_for, search_for_w, (BUFSIZE));
		break;
	case 'E': // EUC-JP
		result = WideCharToMultiByte(0, 0, search_for_w, -1, search_for_sjis, (BUFSIZE), 0, 0);
		// search_for remains null string if it fails.
		if(result == 0) break;
		// sjis2euc_jp is at tools.cpp
		sjis2euc_jp((unsigned char *)search_for, (const unsigned char *)search_for_sjis, strlen(search_for_sjis));
		break;
	case 'L': // UTF-16 to ISO 8859-1 Latin 1
		utf16toLatin1(search_for, search_for_w, (BUFSIZE));
		break;
	default:
		break;
	}
	
	// Trim whitespaces at end of string.
	trim(search_for);

	url_len = strlen(search_url);
	url_len2 = strlen(add_url);

	if(url_len + url_len2 >= (BUFSIZE)) return; // just in case.

	// Use null string if URL is too long.
	if(url_len + url_len2 + strlen(search_for) * 3 >= (BUFSIZE)) 
		*search_for = '\0';

	// Attach encoded 'search_for' string to 'search_url'.
	strcpy(buffer, search_url);
	UrlEncode(buffer + url_len, search_for);
	sprintf(buffer2, "%s%s", buffer, add_url);

	// Open window for the URL.
	ShellExecute(NULL, "open", buffer2, NULL, NULL, SW_SHOWNORMAL);
}

// google.co.jp
void WINAPI nodokagoogle_jpW(const wchar_t *search_for_w){
	char url[] = "http://www.google.co.jp/search?hl=ja&lr=lang_ja&q=";
	searchit(url, search_for_w, 'U');
}

// google.com
void WINAPI nodokagoogleW(const wchar_t *search_for_w){
	char url[] = "http://www.google.com/search?hl=en&q=";
	searchit(url, search_for_w, 'U');
}

// yahoo.co.jp dictionary
void WINAPI nodokayahoodic_jpW(const wchar_t *search_for_w){
	char url[] = "http://dic.yahoo.co.jp/dsearch?stype=0&dtype=2&p=";
	searchit(url, search_for_w, 'E');
}

// wikipedia-jp
void WINAPI nodokawikipedia_jpW(const wchar_t *search_for_w){
	char url[] = "http://ja.wikipedia.org/wiki/";
	searchit(url, search_for_w, 'U');
}

// wikipedia-en
void WINAPI nodokawikipedia_enW(const wchar_t *search_for_w){
	char url[] = "http://en.wikipedia.org/wiki/";
	searchit(url, search_for_w, 'U');
}

// PHP manual
void WINAPI nodokaphpmanW(const wchar_t *search_for_w){
	char url[] = "http://www.php.net/manual-lookup.php?pattern=";
	searchit(url, search_for_w, 'U');
}

// Google Maps-jp
void WINAPI nodokagmaps_jpW(const wchar_t *search_for_w){
	char url[] = "http://maps.google.co.jp/maps?f=q&hl=ja&q=";
	searchit(url, search_for_w, 'U');
}

// Google Maps
void WINAPI nodokagmapsW(const wchar_t *search_for_w){
	char url[] = "http://maps.google.com/maps?hl=en&ie=UTF8&q=";
	searchit(url, search_for_w, 'U');
}

// CPAN
void WINAPI nodokacpanW(const wchar_t *search_for_w){
	char url[] = "http://search.cpan.org/search?mode=all&query=";
	searchit(url, search_for_w, 'U');
}

// goo dictionary
void WINAPI nodokagoodicW(const wchar_t *search_for_w){
	char url[] = "http://dictionary.goo.ne.jp/search.php?kind=all&kwassist=0&mode=0&MT=";
	searchit(url, search_for_w, 'E');
}

// Linux Japanese man pages
void WINAPI nodokalinuxman_jpW(const wchar_t *search_for_w){
	char url[] = "http://search.linux.or.jp/cgi-bin/JM/man.cgi?submit=search%21&Sec1=on&Sec2=on&Sec3=on&Sec4=on&Sec5=on&Sec6=on&Sec7=on&Sec8=on&Pagename=";
	searchit(url, search_for_w, 'U');
}

// kakaku.com
void WINAPI nodokakakakucomW(const wchar_t *search_for_w){
	char url[] = "http://kakaku.com/ksearch/search.asp?LQ=s&C1=&C2=&L=l&Query=";
	searchit(url, search_for_w, 'S');
}

// mapion
void WINAPI nodokamapionW(const wchar_t *search_for_w){
	char url[] = "http://www.mapion.co.jp/c/f?p=1&uc=7&mx=999&vp=20&grp=all&oi=admi3code&ob=1&find=";
	searchit(url, search_for_w, 'S');
}

// www.yahoo.co.jp
void WINAPI nodokayahoo_jpW(const wchar_t *search_for_w){
	char url[] = "http://search.yahoo.co.jp/search?ei=UTF-8&fr=top_v2&x=wrt&p=";
	searchit(url, search_for_w, 'U');
}

// www.yahoo.com
void WINAPI nodokayahooW(const wchar_t *search_for_w){
	char url[] = "http://search.yahoo.com/search?fr=yfp-t-501&toggle=1&cop=mss&ei=UTF-8&p=";
	searchit(url, search_for_w, 'U');
}

// www.amazon.co.jp
void WINAPI nodokaamazon_jpW(const wchar_t *search_for_w){
	char url[] = "http://www.amazon.co.jp/s?__mk_ja_JP=%83J%83%5E%83J%83i&Go=Go&field-keywords=";
	searchit(url, search_for_w, 'S');
}

// www.amazon.com
void WINAPI nodokaamazonW(const wchar_t *search_for_w){
	char url[] = "http://www.amazon.com/s?Go=Go&field-keywords=";
	searchit(url, search_for_w, 'U');
}

// www.msn.co.jp
void WINAPI nodokamsn_jpW(const wchar_t *search_for_w){
	char url[] = "http://search.msn.co.jp/results.aspx?FORM=MSNH&cp=65001&q=";
	searchit(url, search_for_w, 'U');
}

// www.msn.com
void WINAPI nodokamsnW(const wchar_t *search_for_w){
	char url[] = "http://search.msn.com/results.aspx?FORM=MSNH&q=";
	searchit(url, search_for_w, 'U');
}

// www.eminori.com/drug (medicine search)
void WINAPI nodokaeminori_drugW(const wchar_t *search_for_w){
	char url[] = "http://www.eminori.com/mdb/SIKTBL_N02_2080302.asp?Operator=1&%90%B3%8E%AE%96%BC=";
	searchit(url, search_for_w, 'S');
}

// find.2ch.net
void WINAPI nodokafind2chW(const wchar_t *search_for_w){
	char url[] = "http://find.2ch.net/?COUNT=10&TYPE=TITLE&BBS=ALL&STR=";
	searchit(url, search_for_w, 'E');
}

// listen.jp artist search
void WINAPI nodokalistenjp_artistW(const wchar_t *search_for_w){
	char url[] = "http://listen.jp/store/search.aspx?idx=artist&key=";
	searchit(url, search_for_w, 'S');
}

// listen.jp album search
void WINAPI nodokalistenjp_albumW(const wchar_t *search_for_w){
	char url[] = "http://listen.jp/store/search.aspx?idx=album&key=";
	searchit(url, search_for_w, 'S');
}

// ewords_jp
void WINAPI nodokaewords_jpW(const wchar_t *search_for_w){
	char url[] = "http://e-words.jp/?hl=ja&lr=lang_ja&ie=Shift_JIS&oe=Shift_JIS&inlang=ja&q=";
	searchit(url, search_for_w, 'S');
}

// youtube
void WINAPI nodokayoutubeW(const wchar_t *search_for_w){
	char url[] = "http://www.youtube.com/results?search=Search&search_query=";
	searchit(url, search_for_w, 'U');
}

// www.dict.cc
void WINAPI nodokadictccW(const wchar_t *search_for_w){
	char url[] = "http://www.dict.cc/?s=";
	searchit(url, search_for_w, 'U');
}

// dict.tu-chemnitz.de
void WINAPI nodokadicttuchemnitzdeW(const wchar_t *search_for_w){
	char url[] = "http://dict.tu-chemnitz.de/dings.cgi?lang=en&service=deen&opterrors=0&optpro=0&query=";
	searchit(url, search_for_w, 'L');
}

// dict.leo.org
void WINAPI nodokadictleoorgW(const wchar_t *search_for_w){
	char url[] = "http://dict.leo.org/ende?lp=ende&lang=de&searchLoc=0&cmpType=relaxed&sectHdr=on&spellToler=on&chinese=both&pinyin=diacritic&search=";
	searchit(url, search_for_w, 'U');
}

// www.yourdictionary.com
void WINAPI nodokayourdictionarycomW(const wchar_t *search_for_w){
	char url[] = "http://www.yourdictionary.com/search?ydQ=";
	searchit(url, search_for_w, 'U');
}

// www.wadoku.de
void WINAPI nodokawadokudeW(const wchar_t *search_for_w){
	char url[] = "http://www.wadoku.de/index.jsp?search=search&phrase=";
	searchit(url, search_for_w, 'U');
}

// www.duden.de
void WINAPI nodokadudendeW(const wchar_t *search_for_w){
	char url[] = "http://www.duden.de/suche/index.php?begriff=";
	char url2[] = "&bereich=mixed&pneu=";
	searchit2(url, url2, search_for_w, 'L');
}

// www.woerterbuch.info
void WINAPI nodokawoerterbuchinfoW(const wchar_t *search_for_w){
	char url[] = "http://www.woerterbuch.info/?query=";
	searchit(url, search_for_w, 'L');
}

// www.onelook.com
void WINAPI nodokaonelookcomW(const wchar_t *search_for_w){
	char url[] = "http://www.onelook.com/?w=";
	searchit(url, search_for_w, 'U');
}

void WINAPI nodokauser_aW(const wchar_t *sfw){if(user_url[ 0][0] != NULL) searchit(user_url[ 0], sfw, user_enc[ 0]);}
void WINAPI nodokauser_bW(const wchar_t *sfw){if(user_url[ 1][0] != NULL) searchit(user_url[ 1], sfw, user_enc[ 1]);}
void WINAPI nodokauser_cW(const wchar_t *sfw){if(user_url[ 2][0] != NULL) searchit(user_url[ 2], sfw, user_enc[ 2]);}
void WINAPI nodokauser_dW(const wchar_t *sfw){if(user_url[ 3][0] != NULL) searchit(user_url[ 3], sfw, user_enc[ 3]);}
void WINAPI nodokauser_eW(const wchar_t *sfw){if(user_url[ 4][0] != NULL) searchit(user_url[ 4], sfw, user_enc[ 4]);}
void WINAPI nodokauser_fW(const wchar_t *sfw){if(user_url[ 5][0] != NULL) searchit(user_url[ 5], sfw, user_enc[ 5]);}
void WINAPI nodokauser_gW(const wchar_t *sfw){if(user_url[ 6][0] != NULL) searchit(user_url[ 6], sfw, user_enc[ 6]);}
void WINAPI nodokauser_hW(const wchar_t *sfw){if(user_url[ 7][0] != NULL) searchit(user_url[ 7], sfw, user_enc[ 7]);}
void WINAPI nodokauser_iW(const wchar_t *sfw){if(user_url[ 8][0] != NULL) searchit(user_url[ 8], sfw, user_enc[ 8]);}
void WINAPI nodokauser_jW(const wchar_t *sfw){if(user_url[ 9][0] != NULL) searchit(user_url[ 9], sfw, user_enc[ 9]);}
void WINAPI nodokauser_kW(const wchar_t *sfw){if(user_url[10][0] != NULL) searchit(user_url[10], sfw, user_enc[10]);}
void WINAPI nodokauser_lW(const wchar_t *sfw){if(user_url[11][0] != NULL) searchit(user_url[11], sfw, user_enc[11]);}
void WINAPI nodokauser_mW(const wchar_t *sfw){if(user_url[12][0] != NULL) searchit(user_url[12], sfw, user_enc[12]);}
void WINAPI nodokauser_nW(const wchar_t *sfw){if(user_url[13][0] != NULL) searchit(user_url[13], sfw, user_enc[13]);}
void WINAPI nodokauser_oW(const wchar_t *sfw){if(user_url[14][0] != NULL) searchit(user_url[14], sfw, user_enc[14]);}
void WINAPI nodokauser_pW(const wchar_t *sfw){if(user_url[15][0] != NULL) searchit(user_url[15], sfw, user_enc[15]);}
void WINAPI nodokauser_qW(const wchar_t *sfw){if(user_url[16][0] != NULL) searchit(user_url[16], sfw, user_enc[16]);}
void WINAPI nodokauser_rW(const wchar_t *sfw){if(user_url[17][0] != NULL) searchit(user_url[17], sfw, user_enc[17]);}
void WINAPI nodokauser_sW(const wchar_t *sfw){if(user_url[18][0] != NULL) searchit(user_url[18], sfw, user_enc[18]);}
void WINAPI nodokauser_tW(const wchar_t *sfw){if(user_url[19][0] != NULL) searchit(user_url[19], sfw, user_enc[19]);}
void WINAPI nodokauser_uW(const wchar_t *sfw){if(user_url[20][0] != NULL) searchit(user_url[20], sfw, user_enc[20]);}
void WINAPI nodokauser_vW(const wchar_t *sfw){if(user_url[21][0] != NULL) searchit(user_url[21], sfw, user_enc[21]);}
void WINAPI nodokauser_wW(const wchar_t *sfw){if(user_url[22][0] != NULL) searchit(user_url[22], sfw, user_enc[22]);}
void WINAPI nodokauser_xW(const wchar_t *sfw){if(user_url[23][0] != NULL) searchit(user_url[23], sfw, user_enc[23]);}
void WINAPI nodokauser_yW(const wchar_t *sfw){if(user_url[24][0] != NULL) searchit(user_url[24], sfw, user_enc[24]);}
void WINAPI nodokauser_zW(const wchar_t *sfw){if(user_url[25][0] != NULL) searchit(user_url[25], sfw, user_enc[25]);}
