// tools.cpp

/*
function 'sjis2euc_jp' is from postgresql-8.2.3/euc_jp_and_sjis.c
Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
Portions Copyright (c) 1994, Regents of the University of California
*/

/*
Copyright (c) 2007, Canada, Masakatz <canada-nodokal@kaede.to> All rights reserved.

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the 
following conditions are met:

* Redistributions of source code must retain the above
copyright notice, this list of conditions and the 
following disclaimer.
* Redistributions in binary form must reproduce the 
above copyright notice, this list of conditions and
the following disclaimer in the documentation and/or 
other materials provided with the distribution.
* Any the names of its contributors may not be used to 
endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "stdafx.h"

static char num2hex[16] = 
	{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
	'a', 'b', 'c', 'd', 'e', 'f'};

void UrlEncode(unsigned char *out, unsigned char *in)
	{
	unsigned char *p, *q;
	p = in;
	q = out;

	while (*p) {
		if (isalnum(*p)) {
			*q++ = *p;
			}else{
				*q++ = '%';
				*q++ = num2hex[*p / 16];
				*q++ = num2hex[*p % 16];
			}
		p++;
		}
	*q = '\0';
	}
void UrlEncode(char *out, char *in){
	UrlEncode((unsigned char *)out, (unsigned char *)in);
	}

/* The code below is from postgresql-8.2.3/euc_jp_and_sjis.c
* postgresql is under newer BSD licence.
* The code is from here...
*/

void sjis2euc_jp(unsigned char *p, const unsigned char *sjis, int len)
	{
	int			c1,
		c2,
		k;

	while (len > 0)
		{
		c1 = *sjis;

		if (c1 <= 0x7f){
			/* ASCII */
			if (c1 == 0){
				p = NULL;
				return;
				}
			*p++ = c1;
			sjis++; len--;
			continue;
			}

		if (c1 >= 0xa1 && c1 <= 0xdf){
			/* JIS X0201 (1 byte kana) */
			*p++ = 0x8e;
			*p++ = c1;
			sjis++; len--;
			}else{
				// convert only JIS X0208.
				c2 = sjis[1];
				k  = (c1 << 8) + c2;

				if (k < 0xeb3f){
					/* JIS X0208 */
					*p++ = ((c1 & 0x3f) << 1) + 0x9f     + (c2 > 0x9e);
					*p++ = c2 + ((c2 > 0x9e) ? 2 : 0x60) + (c2 < 0x80);
					sjis += 2; len -= 2;
					}else{
						// not JIS X0208. stop converting.
						*p = '\0';
						return;
					}
			}
		}
	*p = '\0';
	}

/* ..to here (end of code from postgresql) */

int utf162utf8( char *out, const wchar_t *in, int bufsize)
	{
	unsigned short *p;
	int q = 0;
	int wcharlen;

	wcharlen = wcslen(in);
	p = (unsigned short *)in;

	for (; p < (unsigned short *)(in + wcharlen); p++) {
		if(q + 3 >= bufsize) break; // break is to avoid buffer overflow
		if (*p < 0x0080) {
			out[q++] = (char)*p;
			} else if (*p < 0x0800) {
				out[q++] = (char)(0xc0 | ((*p & 0x07C0) >>  6));
				out[q++] = (char)(0x80 | ( *p & 0x003f       ));
			} else {
				out[q++] = (char)(0xe0 | ((*p & 0xf000) >> 12));
				out[q++] = (char)(0x80 | ((*p & 0x0fc0) >>  6));
				out[q++] = (char)(0x80 | ( *p & 0x003f       ));
				}
		}
	out[q] = '\0';
	return q;
	}

int utf16toLatin1( char *out, const wchar_t *in, int bufsize)
	{
	unsigned short *p;
	int q = 0;
	int wcharlen;

	wcharlen = wcslen(in);
	p = (unsigned short *)in;

	for (; p < (unsigned short *)(in + wcharlen); p++) {
		if(q + 3 >= bufsize) break; // break is to avoid buffer overflow
		if (*p < 0x00FF) {
			out[q++] = (char)*p;
			} else if (*p < 0x0800) {
				q += 2;
			} else {
				q += 3;
				}
		}
	out[q] = '\0';
	return q;
	}
